@@ -3,29 +3,58 @@
 # Copyright (C) 2008 - Olivier Lauzanne <olauzanne@gmail.com>
 #
 # Distributed under the BSD license, see LICENSE.txt
-from cssselectpatch import selector_to_xpath
+from .cssselectpatch import JQueryTranslator
+from copy import deepcopy
 from lxml import etree
 import lxml.html
-from copy import deepcopy
-from urlparse import urljoin
+import sys
+
+PY3k = sys.version_info >= (3,)
+
+if PY3k:
+    from urllib.request import urlopen
+    from urllib.parse import urlencode
+    from urllib.parse import urljoin
+    basestring = (str, bytes)
+    unicode = str
+else:
+    from urllib2 import urlopen  # NOQA
+    from urllib import urlencode  # NOQA
+    from urlparse import urljoin  # NOQA
+
+
+def func_globals(f):
+    return f.__globals__ if PY3k else f.func_globals
+
+
+def func_code(f):
+    return f.__code__ if PY3k else f.func_code
+
 
 def fromstring(context, parser=None, custom_parser=None):
     """use html parser if we don't have clean xml
     """
+    if hasattr(context, 'read') and hasattr(context.read, '__call__'):
+        meth = 'parse'
+    else:
+        meth = 'fromstring'
     if custom_parser is None:
         if parser is None:
             try:
-                return [etree.fromstring(context)]
+                result = getattr(etree, meth)(context)
             except etree.XMLSyntaxError:
-                return [lxml.html.fromstring(context)]
-
+                result = getattr(lxml.html, meth)(context)
+            if isinstance(result, etree._ElementTree):
+                return [result.getroot()]
+            else:
+                return [result]
         elif parser == 'xml':
-            custom_parser = etree.fromstring
+            custom_parser = getattr(etree, meth)
         elif parser == 'html':
-            custom_parser = lxml.html.fromstring
+            custom_parser = getattr(lxml.html, meth)
         elif parser == 'soup':
             from  lxml.html import soupparser
-            custom_parser = lxml.html.soupparser.fromstring
+            custom_parser = getattr(soupparser, meth)
         elif parser == 'html_fragments':
             custom_parser = lxml.html.fragments_fromstring
         else:
@@ -34,11 +63,15 @@ def fromstring(context, parser=None, custom_parser=None):
     result = custom_parser(context)
     if type(result) is list:
         return result
+    elif isinstance(result, etree._ElementTree):
+        return [result.getroot()]
     else:
         return [result]
 
+
 def callback(func, *args):
-    return func(*args[:func.func_code.co_argcount])
+    return func(*args[:func_code(func).co_argcount])
+
 
 class NoDefault(object):
     def __repr__(self):
@@ -48,12 +81,14 @@ class NoDefault(object):
 no_default = NoDefault()
 del NoDefault
 
+
 class FlexibleElement(object):
     """property to allow a flexible api"""
     def __init__(self, pget, pset=no_default, pdel=no_default):
         self.pget = pget
         self.pset = pset
         self.pdel = pdel
+
     def __get__(self, instance, klass):
         class _element(object):
             """real element to support set/get/del attr and item and js call
@@ -61,21 +96,25 @@ class FlexibleElement(object):
             def __call__(prop, *args, **kwargs):
                 return self.pget(instance, *args, **kwargs)
             __getattr__ = __getitem__ = __setattr__ = __setitem__ = __call__
+
             def __delitem__(prop, name):
                 if self.pdel is not no_default:
                     return self.pdel(instance, name)
                 else:
                     raise NotImplementedError()
             __delattr__ = __delitem__
+
             def __repr__(prop):
-                return '<flexible_element %s>' % self.pget.func_name
+                return '<flexible_element %s>' % self.pget.__name__
         return _element()
+
     def __set__(self, instance, value):
         if self.pset is not no_default:
             self.pset(instance, value)
         else:
             raise NotImplementedError()
 
+
 class PyQuery(list):
     """The main class
     """
@@ -86,8 +125,11 @@ class PyQuery(list):
         self.parser = kwargs.get('parser', None)
         if 'parser' in kwargs:
             del kwargs['parser']
-        if len(args) >= 1 and isinstance(args[0], basestring) \
-           and args[0].startswith('http://'):
+
+        if len(args) >= 1 and \
+           (not PY3k and isinstance(args[0], basestring) or \
+            PY3k and isinstance(args[0], str)) and \
+           args[0].startswith('http://'):
             kwargs['url'] = args[0]
             if len(args) >= 2:
                 kwargs['data'] = args[1]
@@ -98,24 +140,36 @@ class PyQuery(list):
         else:
             self._parent = no_default
 
+        if 'css_translator' in kwargs:
+            self._translator = kwargs.pop('css_translator')
+        elif self.parser in ('xml',):
+            self._translator = JQueryTranslator(xhtml=True)
+        elif self._parent is not no_default:
+            self._translator = self._parent._translator
+        else:
+            self._translator = JQueryTranslator(xhtml=False)
+
+        namespaces = kwargs.get('namespaces', {})
+        if 'namespaces' in kwargs:
+            del kwargs['namespaces']
+
         if kwargs:
             # specific case to get the dom
             if 'filename' in kwargs:
-                html = file(kwargs['filename']).read()
+                html = open(kwargs['filename'])
             elif 'url' in kwargs:
                 url = kwargs.pop('url')
                 if 'opener' in kwargs:
                     opener = kwargs.pop('opener')
                     html = opener(url)
                 else:
-                    from urllib2 import urlopen
-                    from urllib import urlencode
                     method = kwargs.get('method')
                     data = kwargs.get('data')
                     if type(data) in (dict, list, tuple):
                         data = urlencode(data)
 
-                    if isinstance(method, basestring) and method.lower() == 'get' and data:
+                    if isinstance(method, basestring) and \
+                       method.lower() == 'get' and data:
                         if '?' not in url:
                             url += '?'
                         elif url[-1] not in ('?', '&'):
@@ -123,7 +177,12 @@ class PyQuery(list):
                         url += data
                         data = None
 
-                    html = urlopen(url, data).read()
+                    if data and PY3k:
+                        data = data.encode('utf-8')
+
+                    html = urlopen(url, data)
+                    if not self.parser:
+                        self.parser = 'html'
                 self._base_url = url
             else:
                 raise ValueError('Invalid keyword arguments %s' % kwargs)
@@ -134,9 +193,9 @@ class PyQuery(list):
             # determine context and selector if any
             selector = context = no_default
             length = len(args)
-            if len(args) == 1:
+            if length == 1:
                 context = args[0]
-            elif len(args) == 2:
+            elif length == 2:
                 selector, context = args
             else:
                 raise ValueError("You can't do that." +\
@@ -146,8 +205,9 @@ class PyQuery(list):
             if isinstance(context, basestring):
                 try:
                     elements = fromstring(context, self.parser)
-                except Exception, e:
-                    raise ValueError('%r, %s' % (e, context))
+                except Exception:
+                    raise
+                    raise ValueError(repr(context))
             elif isinstance(context, self.__class__):
                 # copy
                 elements = context[:]
@@ -158,8 +218,9 @@ class PyQuery(list):
 
             # select nodes
             if elements and selector is not no_default:
-                xpath = selector_to_xpath(selector)
-                results = [tag.xpath(xpath) for tag in elements]
+                xpath = self._css_to_xpath(selector)
+                results = [tag.xpath(xpath, namespaces=namespaces) \
+                                                    for tag in elements]
                 # Flatten the results
                 elements = []
                 for r in results:
@@ -167,7 +228,11 @@ class PyQuery(list):
 
         list.__init__(self, elements)
 
-    def __call__(self, *args):
+    def _css_to_xpath(self, selector, prefix='descendant-or-self::'):
+        selector = selector.replace('[@', '[')
+        return self._translator.css_to_xpath(selector, prefix)
+
+    def __call__(self, *args, **kwargs):
         """return a new PyQuery instance
         """
         length = len(args)
@@ -175,9 +240,11 @@ class PyQuery(list):
             raise ValueError('You must provide at least a selector')
         if args[0] == '':
             return self.__class__([])
-        if len(args) == 1 and isinstance(args[0], str) and not args[0].startswith('<'):
+        if len(args) == 1 and \
+           isinstance(args[0], str) and \
+           not args[0].startswith('<'):
             args += (self,)
-        result = self.__class__(*args, **dict(parent=self))
+        result = self.__class__(*args, parent=self, **kwargs)
         return result
 
     # keep original list api prefixed with _
@@ -196,22 +263,33 @@ class PyQuery(list):
     def __str__(self):
         """xml representation of current nodes::
 
-            >>> xml = PyQuery('<script><![[CDATA[ ]></script>', parser='html_fragments')
-            >>> print str(xml)
+            >>> xml = PyQuery(
+            ...   '<script><![[CDATA[ ]></script>', parser='html_fragments')
+            >>> print(str(xml))
             <script>&lt;![[CDATA[ ]&gt;</script>
 
         """
-        return ''.join([etree.tostring(e) for e in self])
+        if PY3k:
+            return ''.join([etree.tostring(e, encoding=str) for e in self])
+        else:
+            return ''.join([etree.tostring(e) for e in self])
+
+    def __unicode__(self):
+        """xml representation of current nodes"""
+        return unicode('').join([etree.tostring(e, encoding=unicode) \
+                                                        for e in self])
 
     def __html__(self):
         """html representation of current nodes::
 
-            >>> html = PyQuery('<script><![[CDATA[ ]></script>', parser='html_fragments')
-            >>> print html.__html__()
+            >>> html = PyQuery(
+            ...   '<script><![[CDATA[ ]></script>', parser='html_fragments')
+            >>> print(html.__html__())
             <script><![[CDATA[ ]></script>
 
         """
-        return ''.join([lxml.html.tostring(e) for e in self])
+        return unicode('').join([lxml.html.tostring(e, encoding=unicode) \
+                                                            for e in self])
 
     def __repr__(self):
         r = []
@@ -224,8 +302,31 @@ class PyQuery(list):
                 r.append('<%s%s%s>' % (el.tag, id, c))
             return '[' + (', '.join(r)) + ']'
         except AttributeError:
-            return list.__repr__(self)
+            if PY3k:
+                return list.__repr__(self)
+            else:
+                for el in self:
+                    if isinstance(el, unicode):
+                        r.append(el.encode('utf-8'))
+                    else:
+                        r.append(el)
+                return repr(r)
 
+    @property
+    def root(self):
+        """return the xml root element
+        """
+        if self._parent is not no_default:
+            return self._parent.getroottree()
+        return self[0].getroottree()
+
+    @property
+    def encoding(self):
+        """return the xml encoding of the root element
+        """
+        root = self.root
+        if root is not None:
+            return self.root.docinfo.encoding
 
     ##############
     # Traversing #
@@ -238,7 +339,7 @@ class PyQuery(list):
         if selector is None:
             results = elements
         else:
-            xpath = selector_to_xpath(selector, 'self::')
+            xpath = self._css_to_xpath(selector, 'self::')
             results = []
             for tag in elements:
                 results.extend(tag.xpath(xpath))
@@ -253,13 +354,20 @@ class PyQuery(list):
         return self.__class__(results, **dict(parent=self))
 
     def parent(self, selector=None):
-        return self._filter_only(selector, [e.getparent() for e in self if e.getparent() is not None], unique = True)
+        return self._filter_only(
+                   selector,
+                   [e.getparent() for e in self if e.getparent() is not None],
+                   unique=True)
 
     def prev(self, selector=None):
-        return self._filter_only(selector, [e.getprevious() for e in self if e.getprevious() is not None])
+        return self._filter_only(
+               selector,
+               [e.getprevious() for e in self if e.getprevious() is not None])
 
     def next(self, selector=None):
-        return self._filter_only(selector, [e.getnext() for e in self if e.getnext() is not None])
+        return self._filter_only(
+               selector,
+               [e.getnext() for e in self if e.getnext() is not None])
 
     def _traverse(self, method):
         for e in self:
@@ -284,9 +392,10 @@ class PyQuery(list):
 
     def nextAll(self, selector=None):
         """
-            >>> d = PyQuery('<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>')
-            >>> d('p:last').nextAll()
-            [<img>]
+        >>> h = '<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>'
+        >>> d = PyQuery(h)
+        >>> d('p:last').nextAll()
+        [<img>]
         """
         return self._filter_only(selector, self._nextAll())
 
@@ -295,40 +404,44 @@ class PyQuery(list):
 
     def prevAll(self, selector=None):
         """
-            >>> d = PyQuery('<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>')
-            >>> d('p:last').prevAll()
-            [<p.hello>]
+        >>> h = '<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>'
+        >>> d = PyQuery(h)
+        >>> d('p:last').prevAll()
+        [<p.hello>]
         """
-        return self._filter_only(selector, self._prevAll(), reverse = True)
+        return self._filter_only(selector, self._prevAll(), reverse=True)
 
     def siblings(self, selector=None):
         """
-            >>> d = PyQuery('<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>')
-            >>> d('.hello').siblings()
-            [<p>, <img>]
-            >>> d('.hello').siblings('img')
-            [<img>]
+         >>> h = '<span><p class="hello">Hi</p><p>Bye</p><img scr=""/></span>'
+         >>> d = PyQuery(h)
+         >>> d('.hello').siblings()
+         [<p>, <img>]
+         >>> d('.hello').siblings('img')
+         [<img>]
+
         """
         return self._filter_only(selector, self._prevAll() + self._nextAll())
 
     def parents(self, selector=None):
         """
-            >>> d = PyQuery('<span><p class="hello">Hi</p><p>Bye</p></span>')
-            >>> d('p').parents()
-            [<span>]
-            >>> d('.hello').parents('span')
-            [<span>]
-            >>> d('.hello').parents('p')
-            []
+        >>> d = PyQuery('<span><p class="hello">Hi</p><p>Bye</p></span>')
+        >>> d('p').parents()
+        [<span>]
+        >>> d('.hello').parents('span')
+        [<span>]
+        >>> d('.hello').parents('p')
+        []
         """
         return self._filter_only(
                 selector,
                 [e for e in self._traverse_parent_topdown()],
-                unique = True
+                unique=True
             )
 
     def children(self, selector=None):
-        """Filter elements that are direct children of self using optional selector.
+        """Filter elements that are direct children of self using optional
+        selector:
 
             >>> d = PyQuery('<span><p class="hello">Hi</p><p>Bye</p></span>')
             >>> d
@@ -343,24 +456,27 @@ class PyQuery(list):
 
     def closest(self, selector=None):
         """
-            >>> d = PyQuery('<div class="hello"><p>This is a <strong class="hello">test</strong></p></div>')
-            >>> d('strong').closest('div')
-            [<div.hello>]
-            >>> d('strong').closest('.hello')
-            [<strong.hello>]
-            >>> d('strong').closest('form')
-            []
+        >>> d = PyQuery(
+        ...  '<div class="hello"><p>This is a '
+        ...  '<strong class="hello">test</strong></p></div>')
+        >>> d('strong').closest('div')
+        [<div.hello>]
+        >>> d('strong').closest('.hello')
+        [<strong.hello>]
+        >>> d('strong').closest('form')
+        []
         """
-        try:
-            current = self[0]
-        except IndexError:
-            current = None
-        while current is not None and not self.__class__(current).is_(selector):
-            current = current.getparent()
-        return self.__class__(current, **dict(parent=self))
+        result = []
+        for current in self:
+            while current is not None and \
+                  not self.__class__(current).is_(selector):
+                current = current.getparent()
+            if current is not None:
+                result.append(current)
+        return self.__class__(result, **dict(parent=self))
 
     def filter(self, selector):
-        """Filter elements in self using selector (string or function).
+        """Filter elements in self using selector (string or function):
 
             >>> d = PyQuery('<p class="hello">Hi</p><p>Bye</p>')
             >>> d('p')
@@ -372,32 +488,35 @@ class PyQuery(list):
             >>> d('p').filter(lambda i: PyQuery(this).text() == 'Hi')
             [<p.hello>]
         """
-        if not callable(selector):
+        if not hasattr(selector, '__call__'):
             return self._filter_only(selector, self)
         else:
             elements = []
             try:
                 for i, this in enumerate(self):
-                    selector.func_globals['this'] = this
+                    func_globals(selector)['this'] = this
                     if callback(selector, i):
                         elements.append(this)
             finally:
-                if 'this' in selector.func_globals:
-                    del selector.func_globals['this']
+                f_globals = func_globals(selector)
+                if 'this' in f_globals:
+                    del f_globals['this']
             return self.__class__(elements, **dict(parent=self))
 
     def not_(self, selector):
-        """Return elements that don't match the given selector.
+        """Return elements that don't match the given selector:
 
             >>> d = PyQuery('<p class="hello">Hi</p><p>Bye</p><div></div>')
             >>> d('p').not_('.hello')
             [<p>]
         """
         exclude = set(self.__class__(selector, self))
-        return self.__class__([e for e in self if e not in exclude], **dict(parent=self))
+        return self.__class__([e for e in self if e not in exclude],
+                              **dict(parent=self))
 
     def is_(self, selector):
-        """Returns True if selector matches at least one current element, else False::
+        """Returns True if selector matches at least one current element, else
+        False:
 
             >>> d = PyQuery('<p class="hello">Hi</p><p>Bye</p><div></div>')
             >>> d('p').eq(0).is_('.hello')
@@ -411,7 +530,7 @@ class PyQuery(list):
         return bool(self.__class__(selector, self))
 
     def find(self, selector):
-        """Find elements using selector traversing down from self::
+        """Find elements using selector traversing down from self:
 
             >>> m = '<p><span><em>Whoah!</em></span></p><p><em> there</em></p>'
             >>> d = PyQuery(m)
@@ -419,11 +538,10 @@ class PyQuery(list):
             [<em>, <em>]
             >>> d('p').eq(1).find('em')
             [<em>]
-
-        ..
         """
-        xpath = selector_to_xpath(selector)
-        results = [child.xpath(xpath) for tag in self for child in tag.getchildren()]
+        xpath = self._css_to_xpath(selector)
+        results = [child.xpath(xpath) for tag in self \
+                        for child in tag.getchildren()]
         # Flatten the results
         elements = []
         for r in results:
@@ -444,7 +562,7 @@ class PyQuery(list):
         ..
         """
         # Use slicing to silently handle out of bounds indexes
-        items = self[index:index+1]
+        items = self[index:index + 1]
         return self.__class__(items, **dict(parent=self))
 
     def each(self, func):
@@ -452,12 +570,13 @@ class PyQuery(list):
         """
         try:
             for i, element in enumerate(self):
-                func.func_globals['this'] = element
+                func_globals(func)['this'] = element
                 if callback(func, i, element) == False:
                     break
         finally:
-            if 'this' in func.func_globals:
-                del func.func_globals['this']
+            f_globals = func_globals(func)
+            if 'this' in f_globals:
+                del f_globals['this']
         return self
 
     def map(self, func):
@@ -480,7 +599,7 @@ class PyQuery(list):
         items = []
         try:
             for i, element in enumerate(self):
-                func.func_globals['this'] = element
+                func_globals(func)['this'] = element
                 result = callback(func, i, element)
                 if result is not None:
                     if not isinstance(result, list):
@@ -488,8 +607,9 @@ class PyQuery(list):
                     else:
                         items.extend(result)
         finally:
-            if 'this' in func.func_globals:
-                del func.func_globals['this']
+            f_globals = func_globals(func)
+            if 'this' in f_globals:
+                del f_globals['this']
         return self.__class__(items, **dict(parent=self))
 
     @property
@@ -685,7 +805,7 @@ class PyQuery(list):
     def hide(self):
         """remove display:none to elements style
 
-            >>> print PyQuery('<div style="display:none;"/>').hide()
+            >>> print(PyQuery('<div style="display:none;"/>').hide())
             <div style="display: none"/>
 
         """
@@ -694,7 +814,7 @@ class PyQuery(list):
     def show(self):
         """add display:block to elements style
 
-            >>> print PyQuery('<div />').show()
+            >>> print(PyQuery('<div />').show())
             <div style="display: block"/>
 
         """
@@ -718,20 +838,28 @@ class PyQuery(list):
         """
         return self.attr('value', value)
 
-    def html(self, value=no_default):
+    def html(self, value=no_default, **kwargs):
         """Get or set the html representation of sub nodes.
 
         Get the text value::
 
             >>> d = PyQuery('<div><span>toto</span></div>')
-            >>> print d.html()
+            >>> print(d.html())
             <span>toto</span>
 
+        Extra args are passed to ``lxml.etree.tostring::
+
+            >>> d = PyQuery('<div><span></span></div>')
+            >>> print(d.html())
+            <span/>
+            >>> print(d.html(method='html'))
+            <span></span>
+
         Set the text value::
 
             >>> d.html('<span>Youhou !</span>')
             [<div>]
-            >>> print d
+            >>> print(d)
             <div><span>Youhou !</span></div>
         """
         if value is no_default:
@@ -742,18 +870,27 @@ class PyQuery(list):
             if not children:
                 return tag.text
             html = tag.text or ''
-            html += ''.join(map(lambda x: etree.tostring(x, encoding=unicode), children))
+            if 'encoding' not in kwargs:
+                kwargs['encoding'] = unicode
+            html += unicode('').join([etree.tostring(e, **kwargs) \
+                                                  for e in children])
             return html
         else:
             if isinstance(value, self.__class__):
-                new_html = str(value)
+                new_html = unicode(value)
             elif isinstance(value, basestring):
                 new_html = value
+            elif not value:
+                new_html = ''
+            else:
+                raise ValueError(type(value))
 
             for tag in self:
                 for child in tag.getchildren():
                     tag.remove(child)
-                root = fromstring('<root>' + new_html + '</root>', self.parser)[0]
+                root = fromstring(
+                            unicode('<root>') + new_html + unicode('</root>'),
+                            self.parser)[0]
                 children = root.getchildren()
                 if children:
                     tag.extend(children)
@@ -765,14 +902,14 @@ class PyQuery(list):
         """Get the html representation of the first selected element::
 
             >>> d = PyQuery('<div><span class="red">toto</span> rocks</div>')
-            >>> print d('span')
+            >>> print(d('span'))
             <span class="red">toto</span> rocks
-            >>> print d('span').outerHtml()
+            >>> print(d('span').outerHtml())
             <span class="red">toto</span>
 
             >>> S = PyQuery('<p>Only <b>me</b> & myself</p>')
-            >>> S('b').outerHtml()
-            '<b>me</b>'
+            >>> print(S('b').outerHtml())
+            <b>me</b>
 
         ..
         """
@@ -783,7 +920,7 @@ class PyQuery(list):
         if e0.tail:
             e0 = deepcopy(e0)
             e0.tail = ''
-        return lxml.html.tostring(e0)
+        return lxml.html.tostring(e0, encoding=unicode)
 
     def text(self, value=no_default):
         """Get or set the text representation of sub nodes.
@@ -791,14 +928,14 @@ class PyQuery(list):
         Get the text value::
 
             >>> doc = PyQuery('<div><span>toto</span><span>tata</span></div>')
-            >>> print doc.text()
+            >>> print(doc.text())
             toto tata
 
         Set the text value::
 
             >>> doc.text('Youhou !')
             [<div>]
-            >>> print doc
+            >>> print(doc)
             <div>Youhou !</div>
 
         """
@@ -833,14 +970,15 @@ class PyQuery(list):
 
     def _get_root(self, value):
         if  isinstance(value, basestring):
-            root = fromstring('<root>' + value + '</root>', self.parser)[0]
+            root = fromstring(unicode('<root>') + value + unicode('</root>'),
+                              self.parser)[0]
         elif isinstance(value, etree._Element):
             root = self.__class__(value)
         elif isinstance(value, PyQuery):
             root = value
         else:
             raise TypeError(
-            'Value must be string, PyQuery or Element. Got %r' %  value)
+            'Value must be string, PyQuery or Element. Got %r' % value)
         if hasattr(root, 'text') and isinstance(root.text, basestring):
             root_text = root.text
         else:
@@ -852,7 +990,7 @@ class PyQuery(list):
         """
         root, root_text = self._get_root(value)
         for i, tag in enumerate(self):
-            if len(tag) > 0: # if the tag has children
+            if len(tag) > 0:  # if the tag has children
                 last_child = tag[-1]
                 if not last_child.tail:
                     last_child.tail = ''
@@ -950,12 +1088,12 @@ class PyQuery(list):
 
     def wrap(self, value):
         """A string of HTML that will be created on the fly and wrapped around
-        each target::
+        each target:
 
             >>> d = PyQuery('<span>youhou</span>')
             >>> d.wrap('<div></div>')
             [<div>]
-            >>> print d
+            >>> print(d)
             <div><span>youhou</span></div>
 
         """
@@ -984,10 +1122,11 @@ class PyQuery(list):
         return self
 
     def wrapAll(self, value):
-        """Wrap all the elements in the matched set into a single wrapper element::
+        """Wrap all the elements in the matched set into a single wrapper
+        element::
 
             >>> d = PyQuery('<div><span>Hey</span><span>you !</span></div>')
-            >>> print d('span').wrapAll('<div id="wrapper"></div>')
+            >>> print(d('span').wrapAll('<div id="wrapper"></div>'))
             <div id="wrapper"><span>Hey</span><span>you !</span></div>
 
         ..
@@ -1029,9 +1168,10 @@ class PyQuery(list):
     def replaceWith(self, value):
         """replace nodes by value
         """
-        if callable(value):
+        if hasattr(value, '__call__'):
             for i, element in enumerate(self):
-                self.__class__(element).before(value(i, element) + (element.tail or ''))
+                self.__class__(element).before(
+                        value(i, element) + (element.tail or ''))
                 parent = element.getparent()
                 parent.remove(element)
         else:
@@ -1065,13 +1205,14 @@ class PyQuery(list):
         return self
 
     def remove(self, expr=no_default):
-        """remove nodes
+        """Remove nodes:
 
-        >>> d = PyQuery('<div>Maybe <em>she</em> does <strong>NOT</strong> know</div>')
-        >>> d('strong').remove()
-        [<strong>]
-        >>> print d
-        <div>Maybe <em>she</em> does   know</div>
+         >>> h = '<div>Maybe <em>she</em> does <strong>NOT</strong> know</div>'
+         >>> d = PyQuery(h)
+         >>> d('strong').remove()
+         [<strong>]
+         >>> print(d)
+         <div>Maybe <em>she</em> does   know</div>
         """
         if expr is no_default:
             for tag in self:
@@ -1094,17 +1235,21 @@ class PyQuery(list):
         return self
 
     class Fn(object):
-        """Hook for defining custom function (like the jQuery.fn)
+        """Hook for defining custom function (like the jQuery.fn):
 
-        >>> PyQuery.fn.listOuterHtml = lambda: this.map(lambda i, el: PyQuery(this).outerHtml())
-        >>> S = PyQuery('<ol>   <li>Coffee</li>   <li>Tea</li>   <li>Milk</li>   </ol>')
-        >>> S('li').listOuterHtml()
-        ['<li>Coffee</li>', '<li>Tea</li>', '<li>Milk</li>']
+        .. sourcecode:: python
+
+         >>> fn = lambda: this.map(lambda i, el: PyQuery(this).outerHtml())
+         >>> PyQuery.fn.listOuterHtml = fn
+         >>> S = PyQuery(
+         ...   '<ol>   <li>Coffee</li>   <li>Tea</li>   <li>Milk</li>   </ol>')
+         >>> S('li').listOuterHtml()
+         ['<li>Coffee</li>', '<li>Tea</li>', '<li>Milk</li>']
 
         """
         def __setattr__(self, name, func):
             def fn(self, *args):
-                func.func_globals['this'] = self
+                func_globals(func)['this'] = self
                 return func(*args)
             fn.__name__ = name
             setattr(PyQuery, name, fn)
@@ -1132,5 +1277,5 @@ class PyQuery(list):
                 raise ValueError('You need a base URL to make your links'
                  'absolute. It can be provided by the base_url parameter.')
 
-        self('a').each(lambda: self(this).attr('href', urljoin(base_url, self(this).attr('href'))))
+        self('a').each(lambda: self(this).attr('href', urljoin(base_url, self(this).attr('href')))) # NOQA
         return self
